from typing import Dict, List


from pie_extended.pipeline.postprocessor.glue import GlueProcessor
from pie_extended.pipeline.postprocessor.proto import ProcessorPrototype


class GreekMorphProcessor(GlueProcessor):
    """

    >>> m = GreekMorphProcessor(head_processor=ProcessorPrototype())
    >>> m.set_tasks(["case", "degree", "gend", "lemma", "mood", "num", "pers", "pos", "tense", "voice"])
    ['lemma', 'pos', 'morph']
    >>> m.get_dict("Θύρσις", ["n", "-", "f", "θύρσις", "-", "s", "-", "n", "-", "-"])
    [{'form': 'Θύρσις', 'lemma': 'θύρσις', 'pos': 'n', 'morph': '-s---fn-'}]

    """

    # Output keys are keys that are given in the end
    OUTPUT_KEYS: List[str] = ["form", "lemma", "pos", "morph"]
    # Glue dicts contains tasks that should merge together subtasks
    GLUE: Dict[str, List[str]] = {
        "morph": ["pers", "num", "tense", "mood", "voice", "gend", "case", "degree"]
    }
    # Glue_char is what is used to glue things together -> Tense=Pres|Person=1
    GLUE_CHAR: str = ""
    # Glue Empty are value to take when all things glued together are empty
    GLUE_EMPTY: Dict[str, str] = {"morph": "--------"}
    # Value that means the current element is empty
    EMPTY_TAG: Dict[str, str] = {
        "case": "-", "degree": "-", "gend": "-", "mood": "-", "num": "-",
        "pers": "-", "pos": "-", "tense": "-", "voice": "-"
    }
    KEEP_EMPTY = True

    def __init__(self, *args, **kwargs):
        super(GreekMorphProcessor, self).__init__(*args, **kwargs)

    def _get_glued(self, glued_task: str, token_dict: Dict[str, str]):
        return token_dict[glued_task]



from ...utils import Metadata, File, get_path

DESC = Metadata(
        "Ancient Greek",
        "grc",
        ["Thibault Clérice", "Enrique Manjavacas"],
        "Ancient Greek Model",
        "https://github.com/hipster-philology/greek-lemmatization-data"
)

DOWNLOADS = [
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/case.tar",
         "case.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/degree.tar",
         "degree.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/gend.tar",
         "gend.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/lemma.tar",
         "lemma.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/mood.tar",
         "mood.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/num.tar",
         "num.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/pers.tar",
         "pers.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/pos.tar",
         "pos.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/tense.tar",
         "tense.tar"),
    File("https://github.com/hipster-philology/greek-lemmatization-data/releases/download/0.0.2/voice.tar",
         "voice.tar")
]


Models = "<{},case><{},degree><{},gend><{},lemma><{},mood><{},num><{},pers><{},pos><{},tense><{},voice>".format(
    get_path("grc", "case.tar"),
    get_path("grc", "degree.tar"),
    get_path("grc", "gend.tar"),
    get_path("grc", "lemma.tar"),
    get_path("grc", "mood.tar"),
    get_path("grc", "num.tar"),
    get_path("grc", "pers.tar"),
    get_path("grc", "pos.tar"),
    get_path("grc", "tense.tar"),
    get_path("grc", "voice.tar")
)
